@@ -1,21 +1,21 @@
 
-# $rcs = ' $Id: Charset.pm,v 1.385 2007/05/17 23:58:14 Daddy Exp $ ' ;
+# $rcs = ' $Id: Charset.pm,v 1.401 2011-11-01 01:02:03 Martin Exp $ ' ;
 
 package I18N::Charset;
 
-require 5.002;
+use strict;
+use warnings;
 
-require Exporter;
+require 5.005;
+
+use base 'Exporter';
 use Carp;
 use IO::String;
 
-# Don't leave home without it:
-use strict;
-
 =head1 NAME
 
 I18N::Charset - IANA Character Set Registry names and Unicode::MapUTF8
-(et al.)  conversion scheme names
+(et al.) conversion scheme names
 
 =head1 SYNOPSIS
 
@@ -32,7 +32,7 @@ I18N::Charset - IANA Character Set Registry names and Unicode::MapUTF8
   $sCharset = libi_charset_name('x-sjis');
   # $sCharset is now 'MS_KANJI' which can be passed to `iconv -f $sCharset ...`
   $sCharset = enco_charset_name('Shift-JIS');
-  # $sCharset is now 'shiftjis' which can be passed to `Encode::from_to()`
+  # $sCharset is now 'shiftjis' which can be passed to Encode::from_to()
 
   I18N::Charset::add_iana_alias('my-japanese' => 'iso-2022-jp');
   I18N::Charset::add_map8_alias('my-arabic' => 'arabic7');
@@ -45,7 +45,7 @@ I18N::Charset - IANA Character Set Registry names and Unicode::MapUTF8
 The C<I18N::Charset> module provides access to the IANA Character Set
 Registry names for identifying character encoding schemes.  It also
 provides a mapping to the character set names used by the
-Unicode::Map8 and Unicode::Map modules.
+Unicode::Map and Unicode::Map8 modules.
 
 So, for example, if you get an HTML document with a META CHARSET="..."
 tag, you can fairly quickly determine what Unicode::MapXXX module can
@@ -67,10 +67,9 @@ functions will always return undef.
 #-----------------------------------------------------------------------
 #	Public Global Variables
 #-----------------------------------------------------------------------
-use vars qw( $VERSION @ISA @EXPORT @EXPORT_OK );
-$VERSION = do { my @r = (q$Revision: 1.385 $ =~ /\d+/g); sprintf "%d."."%03d" x $#r, @r };
-@ISA       = qw( Exporter );
-@EXPORT    = qw( iana_charset_name
+our
+$VERSION = do { my @r = (q$Revision: 1.401 $ =~ /\d+/g); sprintf "%d."."%03d" x $#r, @r };
+our @EXPORT = qw( iana_charset_name
 map8_charset_name
 umap_charset_name
 umu8_charset_name
@@ -80,7 +79,7 @@ libi_charset_name
 enco_charset_name
 mib_to_charset_name charset_name_to_mib
  );
-@EXPORT_OK = qw( add_iana_alias add_map8_alias add_umap_alias add_libi_alias add_enco_alias );
+our @EXPORT_OK = qw( add_iana_alias add_map8_alias add_umap_alias add_libi_alias add_enco_alias );
 
 #-----------------------------------------------------------------------
 #	Private Global Variables
@@ -151,11 +150,11 @@ sub iana_charset_name
   return undef unless $code ne '';
   # $iDebug = ($code =~ m!sjis!);
   # print STDERR " + iana_charset_name($code)..." if $iDebug;
-  my $mib = &_short_to_mib($code);
+  my $mib = _short_to_mib($code);
   return undef unless defined $mib;
   # print STDERR " + mib is ($mib)..." if $iDebug;
   # Make sure this is really a IANA mib:
-  return undef if &_is_dummy($mib);
+  return undef if _is_dummy($mib);
   # print STDERR " + is really iana..." if $iDebug;
   return $hsLongnameOfMIB{$mib};
   } # iana_charset_name
@@ -164,8 +163,8 @@ sub iana_charset_name
 sub _try_list
   {
   my $code = shift;
-  my @asTry = ($code, &_strip($code));
-  push @asTry, &_strip($code) if $code =~ s!\A(x-)+!!;  # try without leading x-
+  my @asTry = ($code, _strip($code));
+  push @asTry, _strip($code) if $code =~ s!\A(x-)+!!;  # try without leading x-
   return @asTry;
   } # _try_list
 
@@ -176,7 +175,7 @@ sub _short_to_mib
   # print STDERR " + _short_to_mib($code)..." if DEBUG;
   my $answer = undef;
  TRY_SHORT:
-  foreach my $sTry (&_try_list($code))
+  foreach my $sTry (_try_list($code))
     {
     my $iMIB = $hsMIBofShortname{$sTry} || 'undef';
     # print STDERR "try($sTry)...$iMIB..." if DEBUG;
@@ -196,7 +195,7 @@ sub _short_to_long
   local $^W = 0;
   my $s = shift;
   # print STDERR " + _short_to_long($s)..." if DEBUG;
-  return $hsLongnameOfMIB{&_short_to_mib($s)};
+  return $hsLongnameOfMIB{_short_to_mib($s)};
   } # _short_to_long
 
 
@@ -219,11 +218,11 @@ sub mime_charset_name
   return undef unless defined $code;
   return undef unless $code ne '';
   # print STDERR " + mime_charset_name($code)..." if DEBUG;
-  my $mib = &_short_to_mib($code);
+  my $mib = _short_to_mib($code);
   return undef unless defined $mib;
   # print STDERR " + mib is ($mib)..." if DEBUG;
   # Make sure this is really an IANA mib:
-  return undef if &_is_dummy($mib);
+  return undef if _is_dummy($mib);
   # print STDERR " + is really iana..." if DEBUG;
   return $hsMIMEofMIB{$mib};
   } # mime_charset_name
@@ -260,7 +259,7 @@ sub _maybe_load_enco # PRIVATE
       {
       # First, see if this already has an IANA mapping:
       my $mib;
-      my $sIana = &iana_charset_name($s);
+      my $sIana = iana_charset_name($s);
       if (!defined $sIana)
         {
         # Create a dummy mib:
@@ -269,14 +268,14 @@ sub _maybe_load_enco # PRIVATE
         } # if
       else
         {
-        $mib = &charset_name_to_mib($sIana);
+        $mib = charset_name_to_mib($sIana);
         $iReal++;
         }
       # At this point we have a mib for this Encode entry.
       $MIBtoENCO{$mib} = $s;
       DEBUG_ENCO && print STDERR " +   mib for enco ==$s== is $mib\n";
-      $hsMIBofShortname{&_strip($s)} = $mib;
-      DEBUG_ENCO && print STDERR " +   assign enco =$s==>$mib\n" if &_is_dummy($mib);
+      $hsMIBofShortname{_strip($s)} = $mib;
+      DEBUG_ENCO && print STDERR " +   assign enco =$s==>$mib\n" if _is_dummy($mib);
       } # foreach ENCODING
     if (DEBUG_ENCO)
       {
@@ -286,7 +285,7 @@ sub _maybe_load_enco # PRIVATE
       print STDERR " +   $iReal real mibs re-used.\n";
       } # if
     $iEncoLoaded = 1;
-    &add_enco_alias('Windows-31J', 'cp932');
+    add_enco_alias('Windows-31J', 'cp932');
     } # if
   else
     {
@@ -296,7 +295,7 @@ sub _maybe_load_enco # PRIVATE
 
 sub _mib_to_enco # PRIVATE
   {
-  &_maybe_load_enco();
+  _maybe_load_enco();
   return $MIBtoENCO{shift()};
   } # _mib_to_enco
 
@@ -305,13 +304,13 @@ sub enco_charset_name
   my $code = shift;
   return undef unless defined $code;
   return undef unless $code ne '';
-  &_maybe_load_enco();
+  _maybe_load_enco();
   my $iDebug = 0; # ($code =~ m!johab!i);
   print STDERR " + enco_charset_name($code)..." if ($iDebug || DEBUG_ENCO);
-  my $mib = &_short_to_mib($code);
+  my $mib = _short_to_mib($code);
   return undef unless defined $mib;
   print STDERR " + mib is ($mib)..." if ($iDebug || DEBUG_ENCO);
-  my $ret = &_mib_to_enco($mib);
+  my $ret = _mib_to_enco($mib);
   print STDERR " + enco is ($ret)..." if ($iDebug || DEBUG_ENCO);
   return $ret;
   } # enco_charset_name
@@ -356,7 +355,7 @@ sub _maybe_load_libi # PRIVATE
  FIND_IANA:
         foreach my $sWord (@asWord)
           {
-          last FIND_IANA if ($sIana = &iana_charset_name($sWord));
+          last FIND_IANA if ($sIana = iana_charset_name($sWord));
           } # foreach FIND_IANA
         if (!defined $sIana)
           {
@@ -365,7 +364,7 @@ sub _maybe_load_libi # PRIVATE
           } # if
         else
           {
-          $mib = &charset_name_to_mib($sIana);
+          $mib = charset_name_to_mib($sIana);
           }
         # At this point we have a mib for this iconv entry.  Assign them all:
  ADD_LIBI:
@@ -373,7 +372,7 @@ sub _maybe_load_libi # PRIVATE
           {
           $MIBtoLIBI{$mib} = $sWord;
           # print STDERR " +   mib for libi ==$sWord== is $mib\n";
-          $hsMIBofShortname{&_strip($sWord)} = $mib;
+          $hsMIBofShortname{_strip($sWord)} = $mib;
           } # foreach ADD_LIBI
         } # foreach ICONV_LINE
       } # if
@@ -383,7 +382,7 @@ sub _maybe_load_libi # PRIVATE
 
 sub _mib_to_libi # PRIVATE
   {
-  &_maybe_load_libi();
+  _maybe_load_libi();
   return $MIBtoLIBI{shift()};
   } # _mib_to_libi
 
@@ -394,10 +393,10 @@ sub libi_charset_name
   return undef unless $code ne '';
   # my $iDebug = 1; # ($code =~ m!johab!i);
   # print STDERR " + libi_charset_name($code)..." if $iDebug;
-  my $mib = &_short_to_mib($code);
+  my $mib = _short_to_mib($code);
   return undef unless defined $mib;
   # print STDERR " + mib is ($mib)..." if $iDebug;
-  my $ret = &_mib_to_libi($mib);
+  my $ret = _mib_to_libi($mib);
   # print STDERR " + libi is ($ret)..." if $iDebug;
   return $ret;
   } # libi_charset_name
@@ -454,11 +453,12 @@ sub charset_name_to_mib
   my $s = shift;
   return undef unless defined($s);
   return $hsMIBofLongname{$s} || $hsMIBofLongname{
-                                      &iana_charset_name($s) ||
-                                      &umap_charset_name($s) ||
-                                      &map8_charset_name($s) ||
-                                      &umu8_charset_name($s) ||
-                                      ''};
+                                                  iana_charset_name($s) ||
+                                                  umap_charset_name($s) ||
+                                                  map8_charset_name($s) ||
+                                                  umu8_charset_name($s) ||
+                                                  ''
+                                                 };
   } # charset_name_to_mib
 
 
@@ -466,10 +466,12 @@ sub charset_name_to_mib
 
 This function takes a string containing the name of a character set
 (in almost any format) and returns a string which contains a name for
-the character set that can be passed to Unicode::Map8::new(). If no
-valid character set name can be identified, then C<undef> will be
-returned.  The case and punctuation within the argument string are not
-important.
+the character set that can be passed to Unicode::Map8::new().
+Note: the returned string will be capitalized just like
+the name of the .bin file in the Unicode::Map8::MAPS_DIR directory.
+If no valid character set name can be identified,
+then C<undef> will be returned.
+The case and punctuation within the argument string are not important.
 
     $sCharset = map8_charset_name('windows-1251');
 
@@ -482,9 +484,9 @@ sub map8_charset_name
   return undef unless $code ne '';
   # $iDebug = 0 && ($code =~ m!037!);
   # print STDERR " + map8_charset_name($code)..." if $iDebug;
-  $code = &_strip($code);
+  $code = _strip($code);
   # print STDERR "$code..." if $iDebug;
-  my $iMIB = &_short_to_mib($code) || 'undef';
+  my $iMIB = _short_to_mib($code) || 'undef';
   # print STDERR "$iMIB..." if $iDebug;
   if ($iMIB ne 'undef')
     {
@@ -495,6 +497,7 @@ sub map8_charset_name
   return undef;
   } # map8_charset_name
 
+
 =item umap_charset_name()
 
 This function takes a string containing the name of a character set
@@ -516,7 +519,7 @@ sub umap_charset_name
   # $iDebug = ($code =~ m!apple!i);
   # print STDERR "\n + MIBtoUMAP{dummymib029} == $MIBtoUMAP{$sDummy .'029'}\n\n" if $iDebug;
   # print STDERR " + umap_charset_name($code)..." if $iDebug;
-  my $iMIB = &_short_to_mib(&_strip($code)) || 'undef';
+  my $iMIB = _short_to_mib(_strip($code)) || 'undef';
   # print STDERR "$iMIB..." if $iDebug;
   if ($iMIB ne 'undef')
     {
@@ -528,6 +531,8 @@ sub umap_charset_name
   } # umap_charset_name
 
 
+our @asMap8Debug;
+
 =item umu8_charset_name()
 
 This function takes a string containing the name of a character set
@@ -548,7 +553,7 @@ sub umu8_charset_name
   return undef unless $code ne '';
   # $iDebug = ($code =~ m!u!);
   # print STDERR " + umu8_charset_name($code)..." if $iDebug;
-  my $iMIB = &_short_to_mib($code) || 'undef';
+  my $iMIB = _short_to_mib($code) || 'undef';
   # print STDERR "$iMIB..." if $iDebug;
   if ($iMIB ne 'undef')
     {
@@ -619,15 +624,15 @@ sub add_iana_alias
   {
   my ($sAlias, $sReal) = @_;
   # print STDERR " + add_iana_alias($sAlias, $sReal)\n";
-  my $sName = &iana_charset_name($sReal);
+  my $sName = iana_charset_name($sReal);
   if (not defined($sName))
     {
     carp qq{attempt to alias "$sAlias" to unknown IANA charset "$sReal"};
     return undef;
     } # if
-  my $mib = &_short_to_mib(&_strip($sName));
+  my $mib = _short_to_mib(_strip($sName));
   # print STDERR " --> $sName --> $mib\n";
-  $hsMIBofShortname{&_strip($sAlias)} = $mib;
+  $hsMIBofShortname{_strip($sAlias)} = $mib;
   return $sName;
   } # add_iana_alias
 
@@ -672,9 +677,9 @@ aliases for 'EBCDIC-CA-FR'.
 sub add_map8_alias
   {
   my ($sAlias, $sReal) = @_;
-  my $sName = &map8_charset_name($sReal);
-  my $sShort = &_strip($sAlias);
-  my $sShortName = &_strip($sName);
+  my $sName = map8_charset_name($sReal);
+  my $sShort = _strip($sAlias);
+  my $sShortName = _strip($sName);
   if (not defined($sName))
     {
     carp qq{attempt to alias "$sAlias" to unknown Map8 charset "$sReal"};
@@ -699,9 +704,9 @@ operates on Unicode::Map encoding tables.
 sub add_umap_alias
   {
   my ($sAlias, $sReal) = @_;
-  my $sName = &umap_charset_name($sReal);
-  my $sShort = &_strip($sAlias);
-  my $sShortName = &_strip($sName);
+  my $sName = umap_charset_name($sReal);
+  my $sShort = _strip($sAlias);
+  my $sShortName = _strip($sName);
   if (not defined($sName))
     {
     carp qq{attempt to alias "$sAlias" to unknown U::Map charset "$sReal"};
@@ -745,15 +750,15 @@ sub add_libi_alias
   {
   my ($sAlias, $sReal) = @_;
   # print STDERR " + add_libi_alias($sAlias,$sReal)...";
-  my $sName = &libi_charset_name($sReal);
+  my $sName = libi_charset_name($sReal);
   if (not defined($sName))
     {
     carp qq{attempt to alias "$sAlias" to unknown iconv charset "$sReal"};
     return undef;
     } # if
-  my $mib = &_short_to_mib(&_strip($sName));
+  my $mib = _short_to_mib(_strip($sName));
   # print STDERR "sName=$sName...mib=$mib\n";
-  $hsMIBofShortname{&_strip($sAlias)} = $mib;
+  $hsMIBofShortname{_strip($sAlias)} = $mib;
   return $sName;
   } # add_libi_alias
 
@@ -789,15 +794,15 @@ sub add_enco_alias
   my ($sAlias, $sReal) = @_;
   my $iDebug = 0;
   print STDERR " + add_enco_alias($sAlias,$sReal)..." if ($iDebug || DEBUG_ENCO);
-  my $sName = &enco_charset_name($sReal);
+  my $sName = enco_charset_name($sReal);
   if (not defined($sName))
     {
     carp qq{attempt to alias "$sAlias" to unknown Encode charset "$sReal"};
     return undef;
     } # if
-  my $mib = &_short_to_mib(&_strip($sName));
+  my $mib = _short_to_mib(_strip($sName));
   print STDERR "sName=$sName...mib=$mib\n" if ($iDebug || DEBUG_ENCO);
-  $hsMIBofShortname{&_strip($sAlias)} = $mib;
+  $hsMIBofShortname{_strip($sAlias)} = $mib;
   return $sName;
   } # add_enco_alias
 
@@ -859,11 +864,9 @@ A wrapper around all three of these character set conversion distributions.
 
 =head1 AUTHOR
 
-Martin Thurn, C<mthurn@cpan.org>, L<http://tinyurl.com/nn67z>.
+Martin 'Kingpin' Thurn, C<mthurn at cpan.org>, L<http://tinyurl.com/nn67z>.
 
-=head1 COPYRIGHT
-
-Copyright (c) 1998-2006 Martin Thurn
+=head1 LICENSE
 
 This module is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
@@ -883,8 +886,7 @@ sub _strip
 
 # The only reason this is a while loop is so that I can bail out
 # (e.g. for debugging) without using goto ;-)
-INFINITE:
-while (1)
+INITIALIZATION:
   {
   my ($sName, $iMIB, $sAlias, $mimename);
   my $iDebug = 0;
@@ -922,7 +924,7 @@ while (1)
       print STDERR " +   found mib: $iMIB, long = $sName\n" if $iDebug;
       $hsLongnameOfMIB{$iMIB} = $sName;
       $hsMIBofLongname{$sName} = $iMIB;
-      $hsMIBofShortname{&_strip($sName)} = $iMIB;
+      $hsMIBofShortname{_strip($sName)} = $iMIB;
       if ($mimename)
         {
         print STDERR " +   found mime: mib=$iMIB, mime=$sName\n" if $iDebug;
@@ -938,7 +940,7 @@ while (1)
       if ($sAlias !~ m!None!i)
         {
         print STDERR " +   map Alias ($sAlias) to mib $iMIB\n" if $iDebug;
-        $hsMIBofShortname{&_strip($sAlias)} = $iMIB;
+        $hsMIBofShortname{_strip($sAlias)} = $iMIB;
 	if ($mimename)
 	  {
           print STDERR " +   found mime: mib=$iMIB, mime=$sAlias\n" if $iDebug;
@@ -951,7 +953,7 @@ while (1)
       } # if Alias
     } # while <$io>
 
-  # last INFINITE;
+  # last;  # for debugging
 
   # Not that we have all the standard definitions, process the special
   # === directives:
@@ -967,7 +969,7 @@ while (1)
     next unless $as[0] ne '';
     next unless defined $as[1];
     next unless $as[1] ne '';
-    my $iMIB = $hsMIBofShortname{&_strip($as[0])} || '';
+    my $iMIB = $hsMIBofShortname{_strip($as[0])} || '';
     unless ($iMIB ne '')
       {
       print STDERR " --- can not find IANA entry for equal-sign directive $as[0]\n";
@@ -975,77 +977,86 @@ while (1)
       } # unless
     foreach my $s (@as)
       {
-      my $sStrip = &_strip($s);
+      my $sStrip = _strip($s);
       # print STDERR " +     $sStrip --> $iMIB\n";
       $hsMIBofShortname{$sStrip} = $iMIB;
       } # foreach
     } # foreach
 
-  # last INFINITE;
+  # last;  # for debugging
 
   if (eval "require Unicode::Map8")
     {
     # $iDebug = 1;
-    print STDERR " + found Unicode::Map8 installed, will build map8 tables..." if $iDebug;
-    my $sMapFile = "$Unicode::Map8::MAPS_DIR/aliases";
-    if (open MAPS, $sMapFile)
+    my $sDir = $Unicode::Map8::MAPS_DIR;
+    my $sAliasesFname = "$sDir/aliases";
+    # Ah, how to get all the Unicode::Map8 supported charsets...  It
+    # sure ain't easy!  The aliases file in the MAPS_DIR has a nice
+    # set of aliases, but since some charsets have no aliases, they're
+    # not listed in the aliases file!  Ergo, we have to read the
+    # aliases file *and* all the file names in the MAPS_DIR!
+    push @asMap8Debug, " DDD found Unicode::Map8 installed, will build map8 tables based on $sAliasesFname and files in that directory...\n";
+    # First, read all the files in the MAPS_DIR folder and register in our local data structures:
+    if (opendir(DIR, $sDir))
       {
-      while (defined (my $sLine = <MAPS>))
+      my @asFname = grep(!/^\.\.?$/, readdir(DIR));
+      foreach my $sLong (@asFname)
         {
-        # $iDebug = ($sLine =~ m!866!);
-        my @as = split(/\s/, $sLine);
+        next unless -f "$Unicode::Map8::MAPS_DIR/$sLong";
+        $sLong =~ s/\.(?:bin|txt)$//;
+        # Try to find the official IANA name for this encoding:
+        push @asMap8Debug, " DDD   looking for $sLong in iana table...\n";
         my $sFound = '';
-        my $iMIB = '';
-        # The "master" Map8 name is in $as[0].  Try to find the
-        # official IANA name for this encoding:
-        my $sMap8 = shift @as;
- MAP8ITEM:
-        foreach my $s ($sMap8, @as)
+        if (defined (my $sTemp = iana_charset_name($sLong)))
           {
-          print STDERR " +      looking for $s in iana table...\n" if $iDebug;
-          if (defined (my $sTemp = &iana_charset_name($s)))
-            {
-            $sFound = $sTemp;
-            last MAP8ITEM;
-            } # if
-          } # foreach
+          $sFound = $sTemp;
+          } # if
         if ($sFound eq '')
           {
           # $iDebug = 1;
           $iMIB = $sFakeMIB++;
-          print STDERR " +   had to use a dummy mib ($iMIB) for U::Map8==$sMap8==\n" if $iDebug;
-          $hsMIBofLongname{$sMap8} = $iMIB;
+          push @asMap8Debug, " DDD   had to use a dummy mib ($iMIB) for U::Map8==$sLong==\n";
+          $hsMIBofLongname{$sLong} = $iMIB;
           } # unless
         else
           {
           $iMIB = $hsMIBofLongname{$sFound};
-          print STDERR " +   found IANA name $sFound ($iMIB) for Map8 entry $sMap8\n" if $iDebug;
+          push @asMap8Debug, " DDD   found IANA name $sFound ($iMIB) for Map8 entry $sLong\n";
           }
-        # $iDebug = ($iMIB =~ m!225[23]!);
         # Make this IANA mib map to this Map8 name:
-        print STDERR " +      map $iMIB to $sMap8 in MIBtoMAP8...\n" if $iDebug;
-        $MIBtoMAP8{$iMIB} = $sMap8;
-        # Map ALL the Map8 aliases to this Map8 master name:
-        foreach my $s ($sMap8, @as)
-          {
-          $s = &_strip($s);
-          print STDERR " +      map $s to $iMIB in hsMIBofShortname...\n" if $iDebug;
-          $hsMIBofShortname{$s} = $iMIB;
-          } # foreach
-        # $iDebug = 0;
-        } # while
-      close MAPS;
-      } # if open
-    else
+        push @asMap8Debug, " DDD      map $iMIB to $sLong in MIBtoMAP8...\n";
+        $MIBtoMAP8{$iMIB} = $sLong;
+        my $s = _strip($sLong);
+        push @asMap8Debug, " DDD      map $s to $iMIB in hsMIBofShortname...\n";
+        $hsMIBofShortname{$s} = $iMIB;
+        } # foreach
+      } # if
+    # Now, go through the Unicode::Map8 aliases hash and process the aliases:
+    my $avoid_warning = keys %Unicode::Map8::ALIASES;
+    while (my ($alias, $charset) = each %Unicode::Map8::ALIASES)
       {
-      carp " --- couldn't open $sMapFile for read" if $iDebug;
-      }
+      my $iMIB = charset_name_to_mib($charset); # qqq
+      my $s = _strip($alias);
+      push @asMap8Debug, " DDD      map $s to $iMIB in hsMIBofShortname...\n";
+      $hsMIBofShortname{$s} = $iMIB;
+      } # while
     # If there are special cases for Unicode::Map8, add them here:
-    # &add_map8_alias("new-name", "existing-name");
-    print STDERR "done.\n" if $iDebug;
+    add_map8_alias('ISO_8859-13:1998', 'ISO_8859-13');
+    add_map8_alias('L 7', 'ISO_8859-13');
+    add_map8_alias('Latin 7', 'ISO_8859-13');
+    add_map8_alias('ISO_8859-15:1998', 'ISO_8859-15');
+    add_map8_alias('L 0', 'ISO_8859-15');
+    add_map8_alias('Latin 0', 'ISO_8859-15');
+    add_map8_alias('L 9', 'ISO_8859-15');
+    add_map8_alias('Latin 9', 'ISO_8859-15');
+    add_map8_alias('ISO-8859-1-Windows-3.1-Latin-1', 'cp1252');
+    add_map8_alias('csWindows31Latin1', 'cp1252');
+    # Above aliases were described in RT#18802
+    push @asMap8Debug, "done.\n";
+    print STDERR @asMap8Debug if $iDebug;
     } # if Unicode::Map8 installed
 
-  # last INFINITE;
+  # last;  # for debugging
 
   # $iDebug = 1;
   if (eval "require Unicode::Map")
@@ -1084,7 +1095,7 @@ while (1)
           foreach my $sAlias ($sName, @asAlias)
             {
             print STDERR " +     try alias $sAlias\n" if $iDebug;
-            my $iMIBtry = &_short_to_mib(&_strip($sAlias));
+            my $iMIBtry = _short_to_mib(_strip($sAlias));
             if ($iMIBtry)
               {
               print STDERR " +       matched\n" if $iDebug;
@@ -1104,11 +1115,11 @@ while (1)
         print STDERR " +   UMAP mib is $iMIB\n" if $iDebug;
         $MIBtoUMAP{$iMIB} = $sName;
         $hsMIBofLongname{$sName} ||= $iMIB;
-        $hsMIBofShortname{&_strip($sName)} ||= $iMIB;
+        $hsMIBofShortname{_strip($sName)} ||= $iMIB;
         foreach my $sAlias (@asAlias)
           {
           print STDERR " +   UMAP alias $sAlias\n" if $iDebug;
-          $hsMIBofShortname{&_strip($sAlias)} = $iMIB;
+          $hsMIBofShortname{_strip($sAlias)} = $iMIB;
           } # foreach $sAlias
         } # foreach UMAP_ENTRY
       close MAPS;
@@ -1119,7 +1130,7 @@ while (1)
       carp " --- couldn't open $sMapFile for read" if $iDebug;
       }
     # If there are special cases for Unicode::Map, add them here:
-    # &add_umap_alias("new-name", "existing-name");
+    # add_umap_alias("new-name", "existing-name");
     print STDERR "done.\n" if $iDebug;
     } # if Unicode::Map installed
 
@@ -1131,7 +1142,7 @@ while (1)
     print STDERR " + found Unicode::MapUTF8 $Unicode::MapUTF8::VERSION installed, will build tables...\n" if $iDebug;
     my @as;
     # Wrap this in an eval to avoid compiler warning(?):
-    eval { @as = &Unicode::MapUTF8::utf8_supported_charset() };
+    eval { @as = Unicode::MapUTF8::utf8_supported_charset() };
 UMU8_NAME:
     foreach my $sName (@as)
       {
@@ -1152,7 +1163,7 @@ UMU8_NAME:
         $MIBtoUMU8{charset_name_to_mib($s)} = $sName;
         next UMU8_NAME;
         } # if maps to U::Map
-      # print STDERR " +   UmapUTF8 entry ===$sName=== has no U::Map entry\n" if $iDebug;
+      # print STDERR " +   UmapUTF8 entry ==$sName== has no U::Map entry\n" if $iDebug;
       $s = map8_charset_name($sName) || '';
       if ($s ne '')
         {
@@ -1160,15 +1171,15 @@ UMU8_NAME:
         $MIBtoUMU8{charset_name_to_mib($s)} = $sName;
         next UMU8_NAME;
         } # if maps to U::Map8
-      print STDERR " +   UmapUTF8 entry ===$sName=== has no entries at all\n" if $iDebug;
+      print STDERR " +   UmapUTF8 entry ==$sName== has no entries at all\n" if $iDebug;
       } # foreach
     # If there are special cases for Unicode::MapUTF8, add them here:
-    # &add_umap_alias("new-name", "existing-name");
+    # add_umap_alias("new-name", "existing-name");
     print STDERR "done.\n" if $iDebug;
     } # if Unicode::MapUTF8 installed
 
-  last INFINITE;
-
+  # Initialization is all finished:
+  last;
   # Below here is debugging code:
 
   print STDERR " + the following IANA names do *not* have entries in the Map8 table:\n";
@@ -1176,17 +1187,17 @@ UMU8_NAME:
   foreach my $sIANA (sort values %hsLongnameOfMIB)
     {
     next if $hiTried{$sIANA};
-    print "$sIANA\n" unless defined &map8_charset_name($sIANA);
+    print "$sIANA\n" unless defined map8_charset_name($sIANA);
     $hiTried{$sIANA}++;
     } # foreach
 
-  # last INFINITE;
+  # last;  # for debugging
 
   # debugging: selective dump:
   print STDERR " + after init, iana_charset_name returns:\n";
   foreach my $key (qw(cp1251 windows-1251 WinCyrillic sjis x-sjis Shift_JIS ASCII US-ASCII us-ascii iso-2022-jp iso-8859-1 Unicode-2-0-utf-8 EUC-KR big5 x-x-big5))
     {
-    print STDERR " +   $key => ", &iana_charset_name($key) || 'undef', "\n";
+    print STDERR " +   $key => ", iana_charset_name($key) || 'undef', "\n";
     } # foreach
 
   # exit 88;
@@ -1194,22 +1205,21 @@ UMU8_NAME:
   print STDERR " + after init, map8_charset_name() returns:\n";
   foreach my $key (qw(cp1251 windows-1251 WinCyrillic sjis x-sjis Shift_JIS ASCII US-ASCII us-ascii iso-2022-jp iso-8859-1 Unicode-2-0-utf-8 EUC-KR big5 x-x-big5))
     {
-    print STDERR " +   $key => ", &map8_charset_name($key) || 'undef', "\n";
+    print STDERR " +   $key => ", map8_charset_name($key) || 'undef', "\n";
     } # foreach
 
-  last INFINITE;
+  last;
 
   # debugging: huge dump:
-  # &_dump_hash('hsLongnameOfMIB', \%hsLongnameOfMIB);
-  # &_dump_hash('hsMIBofLongname', \%hsMIBofLongname);
-  # &_dump_hash('hsMIBofShortname', \%hsMIBofShortname);
+  # _dump_hash('hsLongnameOfMIB', \%hsLongnameOfMIB);
+  # _dump_hash('hsMIBofLongname', \%hsMIBofLongname);
+  # _dump_hash('hsMIBofShortname', \%hsMIBofShortname);
   foreach (keys %hsMIBofShortname)
     {
-    print STDERR " + _short_to_long($_) == ", &_short_to_long($_) || 'undef', "\n";
+    print STDERR " + _short_to_long($_) == ", _short_to_long($_) || 'undef', "\n";
     } # foreach
 
-  last INFINITE;
-  } # while
+  } # end of INITIALIZATION block
 
 sub _dump_hash
   {
@@ -1294,6 +1304,12 @@ koi8-r === cp878
 # ??? === dingbats
 # ??? === nextstep
 # ??? === posix-bc
+# The following aliases are listed in RT#18802:
+ISO-8859-10 === 8859-10 === ISO_8859-10:1993
+# TCVN-5712 x-viet-tcvn viet-tcvn VN-1 TCVN-5712:1993
+TIS-620 === TIS_620-2553 === TIS_620-2553:1990
+# VPS x-viet-vps viet-vps
+# The above aliases are listed in RT#18802
 XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
   } # _init_data_extra
 
@@ -1304,10 +1320,9 @@ sub _init_data
   # http://www.iana.org/assignments/character-sets
   return <<'EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE';
 
-===================================================================
 CHARACTER SETS
 
-(last updated 2007-05-14)
+(last updated 2011-10-30)
 
 These are the official names for character sets that may be used in
 the Internet and may be referred to in Internet documentation.  These
@@ -1349,7 +1364,7 @@ numbers along with suggestive names in order to facilitate applications
 that want to display the names in user interfaces.  The "cs" stands 
 for character set and is provided for applications that need a lower 
 case first letter but want to use mixed case thereafter that cannot 
-contain any special characters, such as underbar ("_") and dash ("-").  
+contain any special characters, such as underbar ("_") and dash ("-").
 
 If the character set is from an ISO standard, its cs alias is the ISO
 standard number or name.  If the character set is not from an ISO
@@ -1946,6 +1961,7 @@ Source: ECMA registry
 Alias: iso-ir-121
 Alias: ISO646-CA
 Alias: csa7-1
+Alias: csa71
 Alias: ca
 Alias: csISO121Canadian1
 
@@ -1955,6 +1971,7 @@ Source: ECMA registry
 Alias: iso-ir-122
 Alias: ISO646-CA2
 Alias: csa7-2
+Alias: csa72
 Alias: csISO122Canadian2
 
 Name: CSA_Z243.4-1985-gr                                 [RFC1345,KXS2]
@@ -2227,7 +2244,9 @@ Alias: csUnicodeLatin1
 Alias: ISO-10646
 
 Name: ISO-10646-J-1
+MIBenum: 1004
 Source: ISO 10646 Japanese, see RFC 1815.
+Alias: csUnicodeJapanese
 
 Name: ISO-Unicode-IBM-1261
 MIBenum: 1005
@@ -2865,7 +2884,7 @@ Alias: csKOI8R
 
 Name: HZ-GB-2312
 MIBenum: 2085
-Source: RFC 1842, RFC 1843                                               [RFC1842, RFC1843]
+Source: RFC 1842, RFC 1843                                       [RFC1842, RFC1843]
 
 Name: IBM866                                                     [Pond]
 MIBenum: 2086
@@ -2997,7 +3016,7 @@ Alias:  Amiga1251
 Alias:  Ami-1251
 (Aliases are provided for historical reasons and should not be used)
                                                               [Malyshev]
-															  
+
 Name:  KOI7-switched
 MIBenum:  2105
 Source:  See <http://www.iana.org/assignments/charset-reg/KOI7-switched>
@@ -3013,6 +3032,16 @@ MIBenum: 2107
 Source: See <http://www.iana.org/assignments/charset-reg/TSCII>           [Kalyanasundaram]
 Alias: csTSCII
 
+Name: CP51932
+MIBenum: 2108
+Source: See <http://www.iana.org/assignments/charset-reg/CP51932>                  [Naruse]
+Alias: csCP51932
+
+Name: windows-874
+MIBenum: 2109
+Source: See <http://www.iana.org/assignments/charset-reg/windows-874>              [Steele]
+Alias: None
+
 Name: windows-1250
 MIBenum: 2250
 Source: Microsoft  (http://www.iana.org/assignments/charset-reg/windows-1250) [Lazhintseva]
@@ -3062,6 +3091,11 @@ Name: TIS-620
 MIBenum: 2259
 Source: Thai Industrial Standards Institute (TISI)                             [Tantsetthi]
 
+Name: CP50220
+MIBenum: 2260
+Source: See <http://www.iana.org/assignments/charset-reg/CP50220>                  [Naruse]
+Alias: csCP50220
+
 
 REFERENCES
 ----------
@@ -3133,8 +3167,8 @@ REFERENCES
            Stanford University, August 1995.
 
 [RFC2152]  Goldsmith, D., M. Davis, "UTF-7: A Mail-Safe Transformation
-	   Format of Unicode", RFC 2152, Apple Computer, Inc.,
-	   Taligent Inc., May 1997.
+           Format of Unicode", RFC 2152, Apple Computer, Inc.,
+           Taligent Inc., May 1997.
 
 [RFC2279]  Yergeau, F., "UTF-8, A Transformation Format of ISO 10646",
            RFC 2279, Alis Technologies, January, 1998.
@@ -3166,6 +3200,8 @@ PEOPLE
 
 [Murai] Jun Murai <jun&wide.ad.jp>
 
+[Naruse] Yui Naruse, <naruse&airemix.jp>, 15 September 2010, 23 September 2011. 
+
 [Nussbacher] Hank Nussbacher, <hank&vm.tau.ac.il>
 
 [Ohta] Masataka Ohta, <mohta&cc.titech.ac.jp>, July 1995.
@@ -3181,6 +3217,8 @@ PEOPLE
 
 [Simonsen] Keld Simonsen, <Keld.Simonsen&rap.dk>, August 2000.
 
+[Steele] Shawn Steele, <Shawn.Steele&microsoft.com>, 4 November 2010.
+
 [Tantsetthi] Trin Tantsetthi, <trin&mozart.inet.co.th>, September 1998.
 
 [Thibault] Samuel Thibault, <samuel.thibault&ens-lyon.org>, 7 December 2006.
@@ -3203,6 +3241,7 @@ PEOPLE
 
 
 
+
 EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
   } # _init_data
 
